@@ -29,6 +29,7 @@
 #include "ksaneviewer.h"
 
 #include "selectionitem.h"
+#include "hiderectitem.h"
 
 #include <QGraphicsPixmapItem>
 #include <QGraphicsScene>
@@ -65,11 +66,15 @@ struct KSaneViewer::Private {
     QAction *zoom2FitAction;
     QAction *clrSelAction;
 
-    QGraphicsRectItem *hideLeft;
-    QGraphicsRectItem *hideRight;
-    QGraphicsRectItem *hideTop;
-    QGraphicsRectItem *hideBottom;
-    QGraphicsRectItem *hideArea;
+    HideRectItem *hideLeft;
+    HideRectItem *hideRight;
+    HideRectItem *hideTop;
+    HideRectItem *hideBottom;
+    HideRectItem *hideArea;
+
+    bool multiSelectionEnabled = true;
+
+    int wheelDelta = 0;
 };
 
 KSaneViewer::KSaneViewer(QImage *img, QWidget *parent) : QGraphicsView(parent), d(new Private)
@@ -81,8 +86,9 @@ KSaneViewer::KSaneViewer(QImage *img, QWidget *parent) : QGraphicsView(parent),
     setMouseTracking(true);
 
     // Init the scene
-    d->scene = new QGraphicsScene;
-    d->scene->setSceneRect(0, 0, img->width(), img->height());
+    d->scene = new QGraphicsScene(this);
+    const auto dpr = img->devicePixelRatio();
+    d->scene->setSceneRect(0, 0, img->width() / dpr, img->height() / dpr);
     setScene(d->scene);
 
     d->selection = new SelectionItem(QRectF());
@@ -93,29 +99,13 @@ KSaneViewer::KSaneViewer(QImage *img, QWidget *parent) : QGraphicsView(parent),
     d->selection->setRect(d->scene->sceneRect());
     d->selection->setVisible(false);
 
-    d->hideTop = new QGraphicsRectItem;
-    d->hideBottom = new QGraphicsRectItem;
-    d->hideRight = new QGraphicsRectItem;
-    d->hideLeft = new QGraphicsRectItem;
-    d->hideArea = new QGraphicsRectItem;
-
-    d->hideTop->setOpacity(0.4);
-    d->hideBottom->setOpacity(0.4);
-    d->hideRight->setOpacity(0.4);
-    d->hideLeft->setOpacity(0.4);
+    d->hideTop = new HideRectItem;
+    d->hideBottom = new HideRectItem;
+    d->hideRight = new HideRectItem;
+    d->hideLeft = new HideRectItem;
+    d->hideArea = new HideRectItem;
     d->hideArea->setOpacity(0.6);
 
-    d->hideTop->setPen(Qt::NoPen);
-    d->hideBottom->setPen(Qt::NoPen);
-    d->hideRight->setPen(Qt::NoPen);
-    d->hideLeft->setPen(Qt::NoPen);
-    d->hideArea->setPen(Qt::NoPen);
-
-    d->hideTop->setBrush(QBrush(Qt::black));
-    d->hideBottom->setBrush(QBrush(Qt::black));
-    d->hideRight->setBrush(QBrush(Qt::black));
-    d->hideLeft->setBrush(QBrush(Qt::black));
-
     d->scene->addItem(d->selection);
     d->scene->addItem(d->hideLeft);
     d->scene->addItem(d->hideRight);
@@ -154,7 +144,10 @@ KSaneViewer::KSaneViewer(QImage *img, QWidget *parent) : QGraphicsView(parent),
 void KSaneViewer::drawBackground(QPainter *painter, const QRectF &rect)
 {
     painter->fillRect(rect, QColor(0x70, 0x70, 0x70));
-    painter->drawImage(rect, *d->img, rect);
+    QRectF r = rect & sceneRect();
+    const qreal dpr = d->img->devicePixelRatio();
+    QRectF srcRect = QRectF(r.topLeft() * dpr, r.size() * dpr);
+    painter->drawImage(r, *d->img, srcRect);
 }
 
 // ------------------------------------------------------------------------
@@ -177,11 +170,20 @@ void KSaneViewer::setQImage(QImage *img)
     clearSelections();
 
     // clear zoom
-    setMatrix(QMatrix());
+    resetTransform();
 
-    d->scene->setSceneRect(0, 0, img->width(), img->height());
+    const auto dpr = img->devicePixelRatio();
+    d->scene->setSceneRect(0, 0, img->width() / dpr, img->height() / dpr);
     d->selection->setMaxRight(img->width());
     d->selection->setMaxBottom(img->height());
+
+    d->selection->setDevicePixelRatio(dpr);
+    d->hideTop->setDevicePixelRatio(dpr);
+    d->hideBottom->setDevicePixelRatio(dpr);
+    d->hideRight->setDevicePixelRatio(dpr);
+    d->hideLeft->setDevicePixelRatio(dpr);
+    d->hideArea->setDevicePixelRatio(dpr);
+
     d->img = img;
 }
 
@@ -196,7 +198,7 @@ void KSaneViewer::updateImage()
 // ------------------------------------------------------------------------
 void KSaneViewer::zoomIn()
 {
-    scale(1.5, 1.5);
+    scale(1.3, 1.3);
     d->selection->saveZoom(transform().m11());
     for (int i = 0; i < d->selectionList.size(); ++i) {
         d->selectionList[i]->saveZoom(transform().m11());
@@ -206,7 +208,7 @@ void KSaneViewer::zoomIn()
 // ------------------------------------------------------------------------
 void KSaneViewer::zoomOut()
 {
-    scale(1.0 / 1.5, 1.0 / 1.5);
+    scale(1.0 / 1.3, 1.0 / 1.3);
     d->selection->saveZoom(transform().m11());
     for (int i = 0; i < d->selectionList.size(); ++i) {
         d->selectionList[i]->saveZoom(transform().m11());
@@ -410,8 +412,7 @@ void KSaneViewer::updateHighlight()
 // ------------------------------------------------------------------------
 void KSaneViewer::clearHighlight()
 {
-    d->hideLeft
-    ->hide();
+    d->hideLeft->hide();
     d->hideRight->hide();
     d->hideTop->hide();
     d->hideBottom->hide();
@@ -514,14 +515,29 @@ void KSaneViewer::clearSelections()
     updateSelVisibility();
 }
 
+// ------------------------------------------------------------------------
+void KSaneViewer::setMultiselectionEnabled(bool enabled)
+{
+    d->multiSelectionEnabled = enabled;
+    clearSelections();
+    d->selection->setAddButtonEnabled(enabled);
+}
+
+
 // ------------------------------------------------------------------------
 void KSaneViewer::wheelEvent(QWheelEvent *e)
 {
     if (e->modifiers() == Qt::ControlModifier) {
-        if (e->delta() > 0) {
+        d->wheelDelta +=  e->angleDelta().y();
+
+        while (d->wheelDelta >= QWheelEvent::DefaultDeltasPerStep) {
             zoomIn();
-        } else {
+            d->wheelDelta -= QWheelEvent::DefaultDeltasPerStep;
+        }
+
+        while (d->wheelDelta <= -QWheelEvent::DefaultDeltasPerStep) {
             zoomOut();
+            d->wheelDelta += QWheelEvent::DefaultDeltasPerStep;
         }
     } else {
         QGraphicsView::wheelEvent(e);
@@ -534,7 +550,7 @@ void KSaneViewer::mousePressEvent(QMouseEvent *e)
     if (e->button() == Qt::LeftButton) {
         d->m_left_last_x = e->x();
         d->m_left_last_y = e->y();
-        QPointF scenePoint = mapToScene(e->pos());
+        QPointF scenePoint = scenePos(e) * d->selection->devicePixelRatio();
         d->lastSPoint = scenePoint;
         if (e->modifiers() != Qt::ControlModifier) {
             if (!d->selection->isVisible()) {
@@ -559,11 +575,11 @@ void KSaneViewer::mouseReleaseEvent(QMouseEvent *e)
     if (e->button() == Qt::LeftButton) {
         if ((d->selection->rect().width() < 0.001) ||
                 (d->selection->rect().height() < 0.001)) {
-            emit newSelection(0.0, 0.0, 0.0, 0.0);
+            emit newSelection(0.0, 0.0, 1.0, 1.0);
             clearActiveSelection();
         }
 
-        QPointF scenePoint = mapToScene(e->pos());
+        QPointF scenePoint = scenePos(e) * d->selection->devicePixelRatio();
         for (int i = 0; i < d->selectionList.size(); i++) {
             if (d->selectionList[i]->intersects(scenePoint) == SelectionItem::AddRemove) {
                 d->scene->removeItem(d->selectionList[i]);
@@ -580,6 +596,7 @@ void KSaneViewer::mouseReleaseEvent(QMouseEvent *e)
         if (!removed && (d->selection->intersects(scenePoint) == SelectionItem::AddRemove)) {
             // add the current selection
             SelectionItem *tmp = new SelectionItem(d->selection->rect());
+            tmp->setDevicePixelRatio(d->img->devicePixelRatio());
             d->selectionList.push_back(tmp);
             d->selectionList.back()->setSaved(true);
             d->selectionList.back()->saveZoom(transform().m11());
@@ -588,7 +605,7 @@ void KSaneViewer::mouseReleaseEvent(QMouseEvent *e)
             d->selectionList.back()->intersects(scenePoint);
 
             // clear the old one
-            emit newSelection(0.0, 0.0, 0.0, 0.0);
+            emit newSelection(0.0, 0.0, 1.0, 1.0);
             clearActiveSelection();
         }
     }
@@ -611,7 +628,7 @@ void KSaneViewer::mouseReleaseEvent(QMouseEvent *e)
 // ------------------------------------------------------------------------
 void KSaneViewer::mouseMoveEvent(QMouseEvent *e)
 {
-    QPointF scenePoint = mapToScene(e->pos());
+    QPointF scenePoint = scenePos(e) * d->selection->devicePixelRatio();
 
     if (e->buttons()&Qt::LeftButton) {
         if (e->modifiers() == Qt::ControlModifier) {
@@ -920,6 +937,7 @@ void KSaneViewer::findSelections(float area)
                                 float selArea = (float)(wSelEnd - wSelStart) * (float)(hSelEnd - hSelStart);
                                 if (selArea > (area * MIN_AREA_SIZE)) {
                                     SelectionItem *tmp = new SelectionItem(QRect(QPoint(x1, y1), QPoint(x2, y2)));
+                                    tmp->setDevicePixelRatio(d->img->devicePixelRatio());
                                     d->selectionList.push_back(tmp);
                                     d->selectionList.back()->setSaved(true);
                                     d->selectionList.back()->saveZoom(transform().m11());
@@ -1149,4 +1167,16 @@ int KSaneViewer::refineColumn(int fromCol, int toCol, int rowStart, int rowEnd)
     return col;
 }
 
+QPointF KSaneViewer::scenePos(QMouseEvent *e) const
+{
+    // QGraphicsView::mapToScene() maps only QPoints, but in highdpi mode we want
+    // to deal with non-rounded coordinates, that's why QPainterPath wrapper is used.
+    // QMouseEvent::localPos() currently returns a rounded QPointF
+    // (https://codereview.qt-project.org/259785), so we have to extract a fractional
+    // part from QMouseEvent::screenPos().
+    const QPointF screenPos = e->screenPos();
+    QPointF delta = screenPos - screenPos.toPoint();
+    return mapToScene(QPainterPath(e->pos() + delta)).currentPosition();
+}
+
 }  // NameSpace KSaneIface
