@@ -40,8 +40,8 @@
 #include <QSplitter>
 #include <QMutex>
 #include <QPointer>
-#include <QDebug>
 #include <QIcon>
+#include <QShortcut>
 
 #include <kpassworddialog.h>
 #ifdef HAVE_KF5WALLET
@@ -59,12 +59,14 @@
 #include "ksanedevicedialog.h"
 #include "labeledgamma.h"
 
+#include <ksane_debug.h>
+
 namespace KSaneIface
 {
 static int     s_objectCount = 0;
-static QMutex  s_objectMutex;
+Q_GLOBAL_STATIC(QMutex, s_objectMutex)
 
-static const QString InvetColorsOption = QStringLiteral("KSane::InvertColors");
+static const QLatin1String InvetColorsOption("KSane::InvertColors");
 
 KSaneWidget::KSaneWidget(QWidget *parent)
     : QWidget(parent), d(new KSaneWidgetPrivate(this))
@@ -72,23 +74,23 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     SANE_Int    version;
     SANE_Status status;
 
-    s_objectMutex.lock();
+    s_objectMutex->lock();
     s_objectCount++;
 
     if (s_objectCount == 1) {
         // only call sane init for the first instance
         status = sane_init(&version, &KSaneAuth::authorization);
         if (status != SANE_STATUS_GOOD) {
-            qDebug() << "libksane: sane_init() failed("
+            qCDebug(KSANE_LOG) << "libksane: sane_init() failed("
                      << sane_strstatus(status) << ")";
         } else {
-            //qDebug() << "Sane Version = "
+            //qCDebug(KSANE_LOG) << "Sane Version = "
             //         << SANE_VERSION_MAJOR(version) << "."
             //         << SANE_VERSION_MINORparent(version) << "."
             //         << SANE_VERSION_BUILD(version);
         }
     }
-    s_objectMutex.unlock();
+    s_objectMutex->unlock();
 
     // read the device list to get a list of vendor and model info
     d->m_findDevThread->start();
@@ -159,15 +161,21 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     d->m_clearSelBtn->setToolTip(i18n("Clear Selections"));
     connect(d->m_clearSelBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(clearSelections()));
 
+    QShortcut *prevShortcut = new QShortcut(QKeySequence(QStringLiteral("Ctrl+P")), this);
+    connect(prevShortcut,   SIGNAL(activated()), d, SLOT(startPreviewScan()));
+
+    QShortcut *scanShortcut = new QShortcut(QKeySequence(QStringLiteral("Ctrl+S")), this);
+    connect(scanShortcut,   SIGNAL(activated()), d, SLOT(startFinalScan()));
+
     d->m_prevBtn = new QPushButton(this);
     d->m_prevBtn->setIcon(QIcon::fromTheme(QStringLiteral("document-import")));
-    d->m_prevBtn->setToolTip(i18n("Scan Preview Image"));
+    d->m_prevBtn->setToolTip(i18n("Scan Preview Image (%1)", prevShortcut->key().toString()));
     d->m_prevBtn->setText(i18nc("Preview button text", "Preview"));
     connect(d->m_prevBtn,   SIGNAL(clicked()), d, SLOT(startPreviewScan()));
 
     d->m_scanBtn = new QPushButton(this);
     d->m_scanBtn->setIcon(QIcon::fromTheme(QStringLiteral("document-save")));
-    d->m_scanBtn->setToolTip(i18n("Scan Final Image"));
+    d->m_scanBtn->setToolTip(i18n("Scan Final Image (%1)", scanShortcut->key().toString()));
     d->m_scanBtn->setText(i18nc("Final scan button text", "Scan"));
     d->m_scanBtn->setFocus(Qt::OtherFocusReason);
     connect(d->m_scanBtn,   SIGNAL(clicked()), d, SLOT(startFinalScan()));
@@ -241,11 +249,11 @@ KSaneWidget::KSaneWidget(QWidget *parent)
 KSaneWidget::~KSaneWidget()
 {
     while (!closeDevice()) {
-        usleep(1000);
+        usleep(100);
     }
     // wait for any thread to exit
 
-    s_objectMutex.lock();
+    s_objectMutex->lock();
     s_objectCount--;
     if (s_objectCount <= 0) {
         // only delete the find-devices and authorization singletons and call sane_exit
@@ -254,7 +262,7 @@ KSaneWidget::~KSaneWidget()
         delete d->m_auth;
         sane_exit();
     }
-    s_objectMutex.unlock();
+    s_objectMutex->unlock();
     delete d;
 }
 
@@ -300,10 +308,10 @@ void KSaneWidget::initGetDeviceList() const
 {
     // update the device list if needed to get the vendor and model info
     if (d->m_findDevThread->devicesList().size() == 0) {
-        //qDebug() << "initGetDeviceList() starting thread...";
+        //qCDebug(KSANE_LOG) << "initGetDeviceList() starting thread...";
         d->m_findDevThread->start();
     } else {
-        //qDebug() << "initGetDeviceList() have existing data...";
+        //qCDebug(KSANE_LOG) << "initGetDeviceList() have existing data...";
         d->signalDevListUpdate();
     }
 }
@@ -395,7 +403,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     }
 
     if (status != SANE_STATUS_GOOD) {
-        qDebug() << "sane_open(\"" << deviceName << "\", &handle) failed! status = " << sane_strstatus(status);
+        qCDebug(KSANE_LOG) << "sane_open(\"" << deviceName << "\", &handle) failed! status = " << sane_strstatus(status);
         d->m_auth->clearDeviceAuth(d->m_devName);
         d->m_devName.clear();
         return false;
@@ -461,12 +469,12 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
     // do the connections of the option parameters
     for (i = 0; i < d->m_optList.size(); ++i) {
-        //qDebug() << d->m_optList.at(i)->name();
+        //qCDebug(KSANE_LOG) << d->m_optList.at(i)->name();
         connect(d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect(d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
 
         if (d->m_optList.at(i)->needsPolling()) {
-            //qDebug() << d->m_optList.at(i)->name() << " needs polling";
+            //qCDebug(KSANE_LOG) << d->m_optList.at(i)->name() << " needs polling";
             d->m_pollList.append(d->m_optList.at(i));
             KSaneOptCheckBox *buttonOption = qobject_cast<KSaneOptCheckBox *>(d->m_optList.at(i));
             if (buttonOption) {
@@ -626,7 +634,7 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
     }
     case FormatNone:
     default:
-        qDebug() << "Unsupported conversion";
+        qCDebug(KSANE_LOG) << "Unsupported conversion";
         break;
     }
     float dpm = dpi * (1000.0 / 25.4);
@@ -655,7 +663,7 @@ void KSaneWidget::scanFinal()
         d->startFinalScan();
     } else {
         // if the button frame is disabled, there is no open device to scan from
-        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+        emit scanDone(KSaneWidget::ErrorGeneral, QString());
     }
 }
 
@@ -665,7 +673,7 @@ void KSaneWidget::startPreviewScan()
         d->startPreviewScan();
     } else {
         // if the button frame is disabled, there is no open device to scan from
-        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+        emit scanDone(KSaneWidget::ErrorGeneral, QString());
     }
 }
 
@@ -678,6 +686,7 @@ void KSaneWidget::scanCancel()
     if (d->m_previewThread->isRunning()) {
         d->m_previewThread->cancelScan();
     }
+    d->m_cancelMultiScan = true;
 }
 
 void KSaneWidget::setPreviewResolution(float dpi)
@@ -880,7 +889,7 @@ float KSaneWidget::currentDPI()
             return value;
         }
     }
-    return 0.0;
+    return 0.0; // Failure to read DPI
 }
 
 float KSaneWidget::scanAreaWidth()
@@ -889,7 +898,12 @@ float KSaneWidget::scanAreaWidth()
     if (d->m_optBrX) {
         if (d->m_optBrX->getUnit() == SANE_UNIT_PIXEL) {
             d->m_optBrX->getMaxValue(result);
-            result = result / currentDPI() / 25.4;
+            float dpi = currentDPI();
+            if (dpi < 1) {
+                qCDebug(KSANE_LOG) << "Broken DPI value";
+                dpi = 1.0;
+            }
+            result = result / dpi / 25.4;
         } else if (d->m_optBrX->getUnit() == SANE_UNIT_MM) {
             d->m_optBrX->getMaxValue(result);
         }
@@ -903,7 +917,12 @@ float KSaneWidget::scanAreaHeight()
     if (d->m_optBrY) {
         if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL) {
             d->m_optBrY->getMaxValue(result);
-            result = result / currentDPI() / 25.4;
+            float dpi = currentDPI();
+            if (dpi < 1) {
+                qCDebug(KSANE_LOG) << "Broken DPI value";
+                dpi = 1.0;
+            }
+            result = result / dpi / 25.4;
         } else if (d->m_optBrY->getUnit() == SANE_UNIT_MM) {
             d->m_optBrY->getMaxValue(result);
         }
@@ -916,18 +935,15 @@ void KSaneWidget::setSelection(QPointF topLeft, QPointF bottomRight)
     if (!d->m_optBrX || !d->m_optBrY || !d->m_optTlX || !d->m_optTlY) {
         return;
     }
-    float xmax, ymax;
-    d->m_optBrX->getMaxValue(xmax);
-    d->m_optBrY->getMaxValue(ymax);
     if (topLeft.x() < 0.0 || topLeft.y() < 0.0 || bottomRight.x() < 0.0 || bottomRight.y() < 0.0) {
         d->m_previewViewer->clearActiveSelection();
         return;
     }
 
-    float tlxRatio = topLeft.x()/xmax;
-    float tlyRatio = topLeft.y()/ymax;
-    float brxRatio = bottomRight.x()/xmax;
-    float bryRatio = bottomRight.y()/ymax;
+    float tlxRatio = d->scanAreaToRatioX(topLeft.x());
+    float tlyRatio = d->scanAreaToRatioY(topLeft.y());
+    float brxRatio = d->scanAreaToRatioX(bottomRight.x());
+    float bryRatio = d->scanAreaToRatioX(bottomRight.y());
 
     d->m_previewViewer->setSelection(tlxRatio, tlyRatio, brxRatio, bryRatio);
 }
