@@ -30,7 +30,7 @@
 
 #include "ksaneoptionwidget.h"
 
-#include <QDebug>
+#include <ksane_debug.h>
 
 namespace KSaneIface
 {
@@ -38,8 +38,8 @@ namespace KSaneIface
 KSaneOption::KSaneOption(const SANE_Handle handle, const int index)
     : QObject(), m_handle(handle), m_index(index)
 {
-    m_widget = 0;
-    m_data = 0;
+    m_widget = nullptr;
+    m_data = nullptr;
     readOption();
 }
 
@@ -47,17 +47,17 @@ KSaneOption::~KSaneOption()
 {
     if (m_data) {
         free(m_data);
-        m_data = 0;
+        m_data = nullptr;
     }
     // delete the frame, just in case if no parent is set
     delete m_widget;
-    m_widget = 0;
+    m_widget = nullptr;
 }
 
 void KSaneOption::createWidget(QWidget *parent)
 {
     if (!m_widget) {
-        m_widget = new KSaneOptionWidget(parent, QStringLiteral(""));
+        m_widget = new KSaneOptionWidget(parent, QString());
     }
 
     if (m_optDesc) {
@@ -111,7 +111,7 @@ bool KSaneOption::needsPolling() const
     }
 
     if ((m_optDesc->cap & SANE_CAP_SOFT_DETECT) && !(m_optDesc->cap & SANE_CAP_SOFT_SELECT)) {
-        qDebug() << name() << "optDesc->cap =" << m_optDesc->cap;
+        qCDebug(KSANE_LOG) << name() << "optDesc->cap =" << m_optDesc->cap;
         return true;
     }
 
@@ -120,8 +120,8 @@ bool KSaneOption::needsPolling() const
 
 QString KSaneOption::name() const
 {
-    if (m_optDesc == 0) {
-        return QStringLiteral("");
+    if (m_optDesc == nullptr) {
+        return QString();
     }
     return QString::fromUtf8(m_optDesc->name);
 }
@@ -137,13 +137,13 @@ bool KSaneOption::writeData(void *data)
 
     status = sane_control_option(m_handle, m_index, SANE_ACTION_SET_VALUE, data, &res);
     if (status != SANE_STATUS_GOOD) {
-        qDebug() << m_optDesc->name << "sane_control_option returned:" << sane_strstatus(status);
+        qCDebug(KSANE_LOG) << m_optDesc->name << "sane_control_option returned:" << sane_strstatus(status);
         // write failed. re read the current setting
         readValue();
         return false;
     }
-    if ((res & SANE_INFO_INEXACT) && (m_widget != 0)) {
-        //qDebug() << "write was inexact. Reload value just in case...";
+    if ((res & SANE_INFO_INEXACT) && (m_widget != nullptr)) {
+        //qCDebug(KSANE_LOG) << "write was inexact. Reload value just in case...";
         readValue();
     }
 
@@ -237,13 +237,13 @@ bool KSaneOption::storeCurrentData()
     }
 
     // read that current value
-    if (m_data != 0) {
+    if (m_data != nullptr) {
         free(m_data);
     }
     m_data = (unsigned char *)malloc(m_optDesc->size);
     status = sane_control_option(m_handle, m_index, SANE_ACTION_GET_VALUE, m_data, &res);
     if (status != SANE_STATUS_GOOD) {
-        qDebug() << m_optDesc->name << "sane_control_option returned" << status;
+        qCDebug(KSANE_LOG) << m_optDesc->name << "sane_control_option returned" << status;
         return false;
     }
     return true;
@@ -252,7 +252,7 @@ bool KSaneOption::storeCurrentData()
 bool KSaneOption::restoreSavedData()
 {
     // check if we have saved any data
-    if (m_data == 0) {
+    if (m_data == nullptr) {
         return false;
     }
 
@@ -287,17 +287,17 @@ KSaneOption::KSaneOptType KSaneOption::optionType(const SANE_Option_Descriptor *
             if (optDesc->size == sizeof(SANE_Word)) {
                 return TYPE_SLIDER;
             }
-            qDebug() << "Can not handle:" << optDesc->title;
-            qDebug() << "SANE_CONSTRAINT_NONE && SANE_TYPE_INT";
-            qDebug() << "size" << optDesc->size << "!= sizeof(SANE_Word)";
+            qCDebug(KSANE_LOG) << "Can not handle:" << optDesc->title;
+            qCDebug(KSANE_LOG) << "SANE_CONSTRAINT_NONE && SANE_TYPE_INT";
+            qCDebug(KSANE_LOG) << "size" << optDesc->size << "!= sizeof(SANE_Word)";
             break;
         case SANE_TYPE_FIXED:
             if (optDesc->size == sizeof(SANE_Word)) {
                 return TYPE_F_SLIDER;
             }
-            qDebug() << "Can not handle:" << optDesc->title;
-            qDebug() << "SANE_CONSTRAINT_NONE && SANE_TYPE_FIXED";
-            qDebug() << "size" << optDesc->size << "!= sizeof(SANE_Word)";
+            qCDebug(KSANE_LOG) << "Can not handle:" << optDesc->title;
+            qCDebug(KSANE_LOG) << "SANE_CONSTRAINT_NONE && SANE_TYPE_FIXED";
+            qCDebug(KSANE_LOG) << "size" << optDesc->size << "!= sizeof(SANE_Word)";
             break;
         case SANE_TYPE_BUTTON:
             return TYPE_BUTTON;
@@ -322,22 +322,22 @@ KSaneOption::KSaneOptType KSaneOption::optionType(const SANE_Option_Descriptor *
                     (strcmp(optDesc->name, SANE_NAME_GAMMA_VECTOR_B) == 0)) {
                 return TYPE_GAMMA;
             }
-            qDebug() << "Can not handle:" << optDesc->title;
-            qDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_INT && !SANE_NAME_GAMMA_VECTOR...";
-            qDebug() << "size" << optDesc->size << "!= sizeof(SANE_Word)";
+            qCDebug(KSANE_LOG) << "Can not handle:" << optDesc->title;
+            qCDebug(KSANE_LOG) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_INT && !SANE_NAME_GAMMA_VECTOR...";
+            qCDebug(KSANE_LOG) << "size" << optDesc->size << "!= sizeof(SANE_Word)";
             break;
         case SANE_TYPE_FIXED:
             if (optDesc->size == sizeof(SANE_Word)) {
                 return TYPE_F_SLIDER;
             }
-            qDebug() << "Can not handle:" << optDesc->title;
-            qDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_FIXED";
-            qDebug() << "size" << optDesc->size << "!= sizeof(SANE_Word)";
-            qDebug() << "Analog Gamma vector?";
+            qCDebug(KSANE_LOG) << "Can not handle:" << optDesc->title;
+            qCDebug(KSANE_LOG) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_FIXED";
+            qCDebug(KSANE_LOG) << "size" << optDesc->size << "!= sizeof(SANE_Word)";
+            qCDebug(KSANE_LOG) << "Analog Gamma vector?";
             break;
         case SANE_TYPE_STRING:
-            qDebug() << "Can not handle:" << optDesc->title;
-            qDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_STRING";
+            qCDebug(KSANE_LOG) << "Can not handle:" << optDesc->title;
+            qCDebug(KSANE_LOG) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_STRING";
             return TYPE_DETECT_FAIL;
         case SANE_TYPE_BUTTON:
             return TYPE_BUTTON;
@@ -368,10 +368,10 @@ KLocalizedString KSaneOption::unitString()
     case SANE_UNIT_NONE:        return KLocalizedString();
     case SANE_UNIT_PIXEL:       return ki18ncp("SpinBox parameter unit", " Pixel", " Pixels");
     case SANE_UNIT_BIT:         return ki18ncp("SpinBox parameter unit", " Bit", " Bits");
-    case SANE_UNIT_MM:          return ki18nc("SpinBox parameter unit (Millimeter)", " mm");
-    case SANE_UNIT_DPI:         return ki18nc("SpinBox parameter unit (Dots Per Inch)", " DPI");
-    case SANE_UNIT_PERCENT:     return ki18nc("SpinBox parameter unit (Percentage)", " %");
-    case SANE_UNIT_MICROSECOND: return ki18nc("SpinBox parameter unit (Microseconds)", " µs");
+    case SANE_UNIT_MM:          return ki18ncp("SpinBox parameter unit (Millimeter)", " mm", " mm");
+    case SANE_UNIT_DPI:         return ki18ncp("SpinBox parameter unit (Dots Per Inch)", " DPI", " DPI");
+    case SANE_UNIT_PERCENT:     return ki18ncp("SpinBox parameter unit (Percentage)", " %", " %");
+    case SANE_UNIT_MICROSECOND: return ki18ncp("SpinBox parameter unit (Microseconds)", " µs", " µs");
     }
     return KLocalizedString();
 }
@@ -379,7 +379,7 @@ KLocalizedString KSaneOption::unitString()
 QString KSaneOption::unitDoubleString()
 {
     switch (m_optDesc->unit) {
-    case SANE_UNIT_NONE:        return QStringLiteral("");
+    case SANE_UNIT_NONE:        return QString();
     case SANE_UNIT_PIXEL:       return i18nc("Double numbers. SpinBox parameter unit", " Pixels");
     case SANE_UNIT_BIT:         return i18nc("Double numbers. SpinBox parameter unit", " Bits");
     case SANE_UNIT_MM:          return i18nc("Double numbers. SpinBox parameter unit (Millimeter)", " mm");
@@ -387,7 +387,7 @@ QString KSaneOption::unitDoubleString()
     case SANE_UNIT_PERCENT:     return i18nc("Double numbers. SpinBox parameter unit (Percentage)", " %");
     case SANE_UNIT_MICROSECOND: return i18nc("Double numbers. SpinBox parameter unit (Microseconds)", " µs");
     }
-    return QStringLiteral("");
+    return QString();
 }
 
 }  // NameSpace KSaneIface
