@@ -65,10 +65,15 @@ public:
     QPointF fixTranslation( QPointF dp);
     QRectF rect();
 
+    qreal devicePixelRatio() const;
+    void setDevicePixelRatio(qreal dpr);
+
+    void setAddButtonEnabled(bool enabled);
+
 public:
     // Graphics Item methods
-    QRectF boundingRect() const Q_DECL_OVERRIDE;
-    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) Q_DECL_OVERRIDE;
+    QRectF boundingRect() const override;
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) override;
 
 private:
     struct Private;
